from cdo import *
import sets
import numpy


def settaxis(dataname, varname, f, d):
    cdo=Cdo()
    """test if the calendar is well written and return a another file 
    with the rigth time axis if needed
    - dataname: the name the experience
    - f: the file to be checked
    - d: the startdate of the file
    """
    #calendar and frequency definition in ensemble case    
    timeinfo = { "EnsCmccSeas":("360days","1day"),
                 "EnsMetfrSeas":("360days","1day"),
                 "EnsEcmwfSeas":("360days","1day"),
                 "EnsUkmoSeas":("360days","1day"),
                 "EnsIfmSeas":("360days","1day") } 
    taxis = (cdo.showtimestamp(input=f)[0]).split()
    if (len(sets.Set(taxis)) == 1):  # and (len(sets.Set(taxis[0:-1]-taxis[1:])) != 1):
        print "Warning: the calendar of the file is wrong"
        try:
            calendar, freq = timeinfo.get(dataname)
        except:
            print "no default value of calendar and frequency has been set for the %s experiment" % dataname
            sys.exit(0)
        print "It has been rewriten with a %s calendar and a %s frequency" % (calendar, freq)
        fileout="taxis_"+(f.split("/"))[-1]
        cdo.settaxis(d, "12:00", freq,  input=cdo.setcalendar(calendar, input=f)
                     , output=fileout, options="-r -f nc")
        print fileout
    return fileout



def write_time(ncdfout, ncdfin,  variables):
    
    
 #read value from input file
    if "reftime" in variables:
        reftimein = ncdfin.variables['reftime']
    else:
        print "reftime dimension is not in the file " + f
        sys.exit(0)
    #create the time dimension
    ncdfout.createDimension('time', None)

    #create the reftime variable
    reftime=ncdfout.createVariable("reftime", "i", ('time',))
    reftime[:] = reftimein
    reftime.units = reftimein.units
    reftime.standard_name = "forecast_reference_time"
    reftime.longname = "forecast reference time"
    reftime.calendar=reftimein.calendar
    
    #create the leadtime variable
    #leadtime=ncdfout.createVariable("leadtime", "i", ('time',))
    #to be changed?
    #leadtime[:] = numpy.array([20209,]*ntimestep)
    #print type(numpy.array([20209,]*ntimestep))
    #leadtime.units = "hours" 
    #leadtime.standard_name = "forecast_period"
    #leadtime.longname = "Time elapsed since the start of the forecast"
    #leadtime.bounds = "time_bnd"


def write_lonlat(ncdfout, ncdfin, variables):

            #read value from input file
    if 'longitude' in variables:
        lonin = ncdfin.variables['longitude']
    else:
        print "longitude dimension is not in the file " + f
        sys.exit(0)
    if 'latitude' in variables:
        latin = ncdfin.variables['latitude']
    else:
        print "latitude dimension is not in the file " + f
        sys.exit(0)

    #create the spatial dimension
    ncdfout.createDimension('longitude', numpy.shape(lonin)[0])
    ncdfout.createDimension('latitude', numpy.shape(latin)[0])

    #create the longitude variable
    lon=ncdfout.createVariable("longitude", "f", ('longitude',))
    lat=ncdfout.createVariable("latitude", "f", ('latitude',))
    lon.data_type = "float"
    lat.data_type = "float"
    lon.units = "degrees_east"
    lat.units = "degrees_north"
    lon.axis = "X"
    lat.axis = "Y"
    lon.standard_name = "longitude"
    lat.standard_name = "latitude"
    lon.topology = "circular" 
    lon.modulo = 360
    lon.valid_min = 0.
    lat.valid_min = -89.
    lon.valid_max = 359.
    lat.valid_max = 89. 
    lon[:]=lonin
    lat[:]=latin


def write_insti(ncdfout,nmember,dataname):
    ncdfout.createDimension('string15', 15)
    dictinsti = { "EnsCmccSeas":"CMCC-Bologna   ",
                  "EnsIfmSeas":"IFM-GEOMAR     ",
                  "EnsMetfrSeas":"Meteo-France   ",
                  "EnsUkmoSeas":"UK Met Office  " , 
                  "EnsEcmwfSeas":"ECMWF          "
                  } 

    if dataname in dictinsti.keys():
        insti=dictinsti.get(dataname)
    else:
        print "The institution is not define"
        print "The Institution is set by default to IC3"
        insti="IC3            "
    institution=ncdfout.createVariable("institution", 'c', ('ensemble','string15',))
    institution.standard_name = "institution"
    institution.long_name = "Institution responsible for the forecast system"
    institution[:] = numpy.array([numpy.array(list(insti)),]*nmember)

def write_id(ncdfout,nmember,dataname):
    ncdfout.createDimension("string4", 4)
    dictid = { "EnsCmccSeas":"2001",
               "EnsIfmSeas":"2001",
               "EnsMetfrSeas":"2001",
               "EnsUkmoSeas":"2001" , 
               "EnsEcmwfSeas":"2001" } 
    if dataname in dictid.keys():
        id=dictid.get(dataname)
    else:
        id=dataname
        
    experiment_id=ncdfout.createVariable("experiment_id", 'c', ('ensemble','string4',))
    experiment_id.standard_name = "experiment_id"
    experiment_id.long_name = "Experiment identifier" 
    experiment_id[:]=numpy.array([numpy.array(list(id)),]*nmember)


def write_realization(ncdfout,nmember):
    realization=ncdfout.createVariable("realization", 'c', ('ensemble',))
    realization.standard_name = "realization"
    realization.long_name = "Number of the simulation in the ensemble" 
    realization[:] = numpy.arange(nmember)


def write_source(ncdfout,nmember,dataname):
    ncdfout.createDimension('string60', 60)
    dictsource = { "EnsCmccSeas":"ECHAM5/OPA8.2, Sys 1, Met 0, ENSEMBLES                      ",
                   "EnsIfmSeas":"ECHAM5 T63L31/MPIOM GR15L40, Sys 1, Met 10, ENSEMBLES       ",
                   "EnsMetfrSeas":"ARPEGEClimate4.6/OPA8.2/GELATO, Sys 0, Met 1, ENSEMBLES     ",
                   "EnsUkmoSeas":"HadGEM2, Sys 1, Met 1, ENSEMBLES                            ", 
                   "EnsEcmwfSeas":"IFS31R1/HOPE-E, Sys 1, Met 1, ENSEMBLES                     ",
                   "b0cm":"EC-Earth2.3.0,NEMOVAR-COMBINE,DFS4.3,ERA40/Int,Atm_SV+O pert"} 
    
    if dataname in dictsource.keys():
        src=dictsource.get(dataname)
    else:
        print "The institution is not define"
        print "There is no default value"
        sys.exit(0)
        
    source=ncdfout.createVariable("source", 'c', ('ensemble','string60',))
    source.standard_name = "source"
    source.long_name = "Method of production of the data"
    source[:] = numpy.array([numpy.array(list(src)),]*nmember)
        
