import sys
import os
import time
import datetime
from cdo import *
import copy
from Scientific.N import *
from Scientific.IO.NetCDF import *
from dateutil.relativedelta import *

class Data:
    """This classe define the set of data for the observation or model
    on which will be calculate the statistics.
    The variables are the following:
    - datanames: names of experiments or observational set (string)
    - varname: name of the variables on which the statitics must be calculated (string)
    - inputdir: directory where are stored the original data (string)
    - outpudir: directory to write the output data (string)
    - filelist: list of file on which the calculations must be done (list of string)
    - sdates: list of startdate associated to each file (list of date)
    - filetype: type of the files in Filelist (string)
    - leadtime: number of timestep in each file (int)
    - freq: frequency between 2 timesteps (timedelta)
    - nmenber: number of member (int)"""
    def __init__(self, dataname, varname):
        self._dataname=dataname
        self._varname=varname
        self._inputdir=""
        self._outputdir=""
        self._filelist=[]
        self._sdates=[]
        self._filetype=""
        self._leadtime=-1
        self._freq=-1
        self._nmenber=-1
    
    
    
    ######## set ########

    def set_dataname(self, dataname):
        self._dataname=dataname

    def set_varname(self, varname):
        self._varname=varname
   
    def set_inputdir(self, inputdir):
        self._inputdir=inputdir

    def set_outputdir(self, outputdir):
        self._outputdir=outputdir

    def set_filelist(self, filelist):
        self._filelist=filelist

    def set_sdates(self, sdates):
        self._sdates=sdates

    def set_filetype(self, filetype):
        self._filetype=filetype

    def set_leadtime(self,leadtime):
        self._leadtime=leadtime

    def set_freq(self,freq):
        self._freq=freq

    def set_nmember(self, nmenber):
        self._nmenber = nmenber

    ######## get ########

    def get_dataname(self):
        return self._dataname

    def get_varname(self):
        return self._varname
   
    def get_inputdir(self):
        return self._inputdir

    def get_outputdir(self):
        return self._outputdir

    def get_filelist(self):
        return self._filelist

    def get_sdates(self):
        return self._sdates

    def get_filetype(self):
        return self._filetype

    def get_leadtime(self):
        return self._leadtime

    def get_freq(self):
        return self._freq

    def get_nmember(self):
        return self._nmenber
    ######## print ########

    def see(self):
        print "dataname: ", self.get_dataname()
        print "varname: ", self.get_varname()
        print "inputdir: ", self.get_inputdir()
        print "outputdir: ", self.get_outputdir()
        print "filelist: ", self.get_filelist()
        print "sdates: ", self.get_sdates()
        print "filetype: ", self.get_filetype()
        print "leadtime: ", self.get_leadtime()
        print "freq: ", self.get_freq()
        print "nmember: ", self.get_nmember()

    ######## set_outputdirAuto ########

    def set_outputdirAuto(self):
        """look for input and output directory corresponding to the dataname"""
        inputdir=self.get_inputdir()
        if "/" in  inputdir:
            #remove the last part of the input directory and replace it by statistics
            outputdir="/".join(inputdir.split("/")[:-3])+"/monthly_statistics_new/"
        else:
            print "The path specify must be an abolute path"
            print "this path is not correct or is empty:"
            print inputdir
            sys.exit(0)
        self.set_outputdir(outputdir)
        if (not os.path.isdir(outputdir)):
            os.mkdir(outputdir)
            print "Created directory:"
            print outputdir
            
    def monthly_percentile(self, percentile):
        """calculate the monthly percentile for all file in filelist"""

        cdo=Cdo()
        
        #get the needed variables from the object
        filelist=list(self.get_filelist())
        sdates=list(self.get_sdates())
        dataname=self.get_dataname()
        varname=self.get_varname()
        freq=self.get_freq()
        
        for i in range(len(filelist)):
            f=filelist[i]
            fileout=(f.split("/")[-1])
            fileout=fileout.split(".")[0]+"_q"+str(percentile)+".nc"
            cdo.monpctl(percentile, input=(f,
                                           cdo.monmin(input=f,options="-r -f nc"),
                                           cdo.monmax(input=f,options="-r -f nc")), 
                        output=fileout,
                        options="-r -f nc")

            filelist[i] = fileout
            #when ncdf4 instaled change name of variables in files


        exp_stat=copy.deepcopy(self)
        exp_stat.set_filelist(filelist)
        #the leadtime is calculated based on the last file
        exp_stat.set_leadtime( len(cdo.showtimestamp(input=fileout)[0].split()))
        exp_stat.set_freq = relativedelta(months=+1)
        exp_stat.set_varname(varname + "_q" + str(percentile))
       
        return exp_stat


    def clim_percentile(self, percentile):
        """calculate the monthly percentile for all file in filelist"""

        cdo=Cdo()
        
        #get the needed variables from the object
        filelist=list(self.get_filelist())
        sdates=list(self.get_sdates())
        dataname=self.get_dataname()
        varname=self.get_varname()
        leadtime=self.get_leadtime()
        freq=self.get_freq()
        

        #calculate the climatological percentile
        
        #check if there is no time overlap between the file
        for i in range(len(sdates)-1):
            if sdates[i] freq*leadtime > sdates[i+1]:
                print "the file in filelist are overlaping"
                print "the climatological percentile can not be calculated"
                sys.exit(0)
            
        for i in range(len(filelist)):
            f=filelist[i]
            d=sdates[i]
            
            #open input and output
            ncdfin=NetCDFFile(f, "r")
            ncdfout=NetCDFFile(ftemp, "w")

            variables=ncdfin.variables.keys()
            allDimNames = ncdfin.dimensions.keys() 

            
            
            


    def select_leadtime(self, leadtime1, leadtime2):
        """select the time period between leadtime1 and leadtime2
        in all files of the DataMod object:
        -leadtime1: in month (timedelta)
        -leadtime2: in month (timedelta)
        the file must have the correct format for cdo
        (run the format_files_nc method in case of failure)
        (to be changed when the right named convention has been set)
        """

        cdo=Cdo()
        
        #get the needed variables from the object
        filelist=list(self.get_filelist())
        sdates=list(self.get_sdates())
        dataname=self.get_dataname()
        freq=self.get_freq()
        
        if leadtime2 < leadtime1:
            print "the second leadtime must be larger than the first one"
            sys.exit(0)

        for i in range(len(filelist)):
            f=filelist[i]
            d=sdates[i]
            
            
            fdate1=d+leadtime1
            fdate2=d+leadtime2

            print "the selected time period is:"
            print fdate1, fdate2
            #calculate the last date of the file
            #  #done directly with the file 
            #  #to avoid the different calendar issues
            lastdate=((cdo.showtimestamp(input=f)[0]).split())[-1]
            lastdate=time.strptime(lastdate,'%Y-%m-%dT%H:%M:%S')
            lastdate=datetime.date(lastdate.tm_year, lastdate.tm_mon, lastdate.tm_mday)

            #check if the time period is smaller the the length of the file
           
            if (fdate2 > lastdate):
                print "the selected time period is not include in the file:"
                print f
                sys.exit(0)


            fileout=(f.split("/")[-1])
            lead1str=str(leadtime1.months).zfill(2)
            lead2str=str(leadtime2.months).zfill(2)
            fileout=fileout.split(".")[0]+"_"+lead1str+"-"+lead2str+"_01-01.nc"
            cdo.seldate(fdate1,fdate2, input=f, output=fileout,options="-r -f nc")
            filelist[i]=fileout
            sdates[i]=fdate1
            

        exp_season=copy.deepcopy(self)
        exp_season.set_filelist(filelist)
        exp_season.set_sdates(sdates)
        #the leadtime is calculated based on the last file
        exp_season.set_leadtime( len(cdo.showtimestamp(input=fileout)[0].split()))
        return exp_season

    def seasonal_percentile(self, percentile):
        """calculate the monthly percentile over the whole period
        for all file in filelist"""

        cdo=Cdo()
        
        #get the needed variables from the object
        filelist=list(self.get_filelist())
        sdates=list(self.get_sdates())
        dataname=self.get_dataname()
        varname=self.get_varname()
        freq=self.get_freq()
        
        for i in range(len(filelist)):
            f=filelist[i]
            fileout=(f.split("/")[-1])
            fileout=fileout.split(".")[0]+"_q"+str(percentile)+".nc"
            cdo.timpctl(percentile, input=(f,
                                           cdo.timmin(input=f,options="-r -f nc"),
                                           cdo.timmax(input=f,options="-r -f nc")), 
                        output=fileout,
                        options="-r -f nc")

            filelist[i] = fileout

        
        exp_stat=copy.deepcopy(self)
        exp_stat.set_filelist(filelist)
        #the leadtime is calculated based on the last file
        exp_stat.set_leadtime( len(cdo.showtimestamp(input=fileout)[0].split()))
        exp_stat.set_freq = relativedelta(months=+1)
        #to be changed when final convention defined!!!!!
        exp_stat.set_varname(varname + "_q" + str(percentile) + "_002-004" )
       
        return exp_stat
